# -*- encoding: utf-8 -*-
require 'spec_helper'

describe Comic do
  before do
    Factory :admin
    @user = Factory( :user_yas)
    @author = @user.author
    @artist = Factory :artist_yas, :author_id => @author.id
  end
  
  describe '検証に於いて' do
    before do
    end
    
    it 'オーソドックスなデータなら通る' do
      @comic = Factory.build :normal_comic, :author_id => @author.id
      @comic.should be_valid
    end
    
    context 'titleを検証するとき' do
      it 'nullなら失敗する' do
        @comic = Factory.build :normal_comic, :author_id => @author.id, :title => nil
        @comic.should_not be_valid
      end
      it '100文字以上なら失敗する' do
        @comic = Factory.build :normal_comic, :author_id => @author.id, :title => 'a'*101
        @comic.should_not be_valid
      end
    end
    context 'widthを検証するとき' do
      it 'nullなら失敗する' do
        @comic = Factory.build :normal_comic, :author_id => @author.id, :width => nil
        @comic.should_not be_valid
      end
      it '0なら失敗する' do
        @comic = Factory.build :normal_comic, :author_id => @author.id, :width => 0
        @comic.should_not be_valid
      end
      it '負でも失敗する' do
        @comic = Factory.build :normal_comic, :author_id => @author.id, :width => -1
        @comic.should_not be_valid
      end
      it '正なら通る' do
        @comic = Factory.build :normal_comic, :author_id => @author.id, :width => 1
        @comic.should be_valid
      end
    end
    context 'heightを検証するとき' do
      it 'nullなら失敗する' do
        @comic = Factory.build :normal_comic, :author_id => @author.id, :height => nil
        @comic.should_not be_valid
      end
      it '0なら失敗する' do
        @comic = Factory.build :normal_comic, :author_id => @author.id, :height => 0
        @comic.should_not be_valid
      end
      it '負でも失敗する' do
        @comic = Factory.build :normal_comic, :author_id => @author.id, :height => -1
        @comic.should_not be_valid
      end
      it '正なら通る' do
        @comic = Factory.build :normal_comic, :author_id => @author.id, :height => 1
        @comic.should be_valid
      end
    end
    context 'visibleを検証するとき' do
      it 'nullなら失敗する' do
        @comic = Factory.build :normal_comic, :author_id => @author.id, :visible => nil
        @comic.should_not be_valid
      end
      it '負なら失敗する' do
        @comic = Factory.build :normal_comic, :author_id => @author.id, :visible => -1
        @comic.should_not be_valid
      end
      it '4以上なら失敗する' do
        @comic = Factory.build :normal_comic, :author_id => @author.id, :visible => 4
        @comic.should_not be_valid
      end
    end
    context 'editableを検証するとき' do
      it 'nullなら失敗する' do
        @comic = Factory.build :normal_comic, :author_id => @author.id, :editable => nil
        @comic.should_not be_valid
      end
      it '負なら失敗する' do
        @comic = Factory.build :normal_comic, :author_id => @author.id, :visible => -1
        @comic.should_not be_valid
      end
      it '4以上なら失敗する' do
        @comic = Factory.build :normal_comic, :author_id => @author.id, :visible => 4
        @comic.should_not be_valid
      end
    end
  end
  
  describe 'データ補充に於いて' do
    before do
      @comic = Comic.new
    end
    
    context '初期値を補充するとき' do
      it '空なら0が補充される' do
        @comic.supply_default
        @comic.editable.should == 0
        @comic.visible.should == 0
      end
      it 'editableが空でないなら変化なし' do
        @comic.editable = 1
        lambda{@comic.supply_default}.should_not change(@comic, :editable)
      end
      it 'visibleが空でないなら変化なし' do
        @comic.visible = 1
        lambda{@comic.supply_default}.should_not change(@comic, :visible)
      end
    end
  end
  
  describe '作者判定に於いて' do
    before do
    end
    it '自作のコミックならyes' do
      comic = Factory :normal_comic, :author_id => @author.id
      comic.own?(@author).should == true
    end
    it '他人のコミックならno' do
      comic = Factory :normal_comic, :author_id => @author.id - 1
      comic.own?(@author).should == false
    end
    it '作家が不明ならno' do
      comic = Factory :normal_comic, :author_id => @author.id
      comic.own?(nil).should == false
    end
  end
  describe '閲覧許可に於いて' do
    before do
    end
    it '自作の公開コミックを見るときは許可する' do
      Comic.any_instance.stub(:own?).and_return(true)
      comic = Factory :normal_comic, :author_id => @author.id
      comic.visible?(@author).should == true
    end
    it '自作のコミックは非公開でも許可する' do
      Comic.any_instance.stub(:own?).and_return(true)
      comic = Factory :hidden_comic, :author_id => @author.id
      comic.visible?(@author).should == true
    end
    it '他人のコミックでも公開コミックなら許可する' do
      Comic.any_instance.stub(:own?).and_return(false)
      comic = Factory :normal_comic, :author_id => @author.id - 1
      comic.visible?(@author).should == true
    end
    it '他人のコミックで非公開コミックなら許可しない' do
      Comic.any_instance.stub(:own?).and_return(false)
      comic = Factory :hidden_comic, :author_id => @author.id - 1
      comic.visible?(@author).should == false
    end
  end
  describe '単体取得に於いて' do
    before do
      @comic = Factory :normal_comic, :author_id => @author.id
    end
    it '指定のコミックを返す' do
      c = Comic.show @comic.id, @author.id
      c.should eq @comic
    end
    context '関連テーブルオプションがないとき' do
      it '作家データだけを含んでいる' do
        r = Comic.show_include_opt
        r.should eq [:author]
      end
    end
    context '関連テーブルオプションでコマを含ませたとき' do
      it '作家データとコマデータを含んでいる' do
        r = Comic.show_include_opt(:include => :panels)
        r.should eq [:author, :panels]
      end
    end
  end
  describe '一覧取得に於いて' do
    before do
      @comic = Factory :normal_comic, :author_id => @author.id
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        Comic.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        Comic.page().should eq 1
      end
      it '0以下の場合は1になる' do
        Comic.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        Comic.page_size('7').should eq 7
      end
      it 'nilの場合はComic.default_page_sizeになる' do
        Comic.page_size().should eq Comic.default_page_size
      end
      it '0以下の場合はComic.default_page_sizeになる' do
        Comic.page_size('0').should eq Comic.default_page_size
      end
      it 'Comic.max_page_sizeを超えた場合はComic.max_page_sizeになる' do
        Comic.page_size('1000').should eq Comic.max_page_size
      end
    end
    it 'リストを返す' do
      c = Comic.list
      c.should eq [@comic]
    end
    it '非公開コミックは(自分のコミックであっても)含んでいない' do
      Factory :hidden_comic, :author_id => @author.id
      c = Comic.list
      c.should eq [@comic]
    end
    it '時系列で並んでいる' do
      v = Factory :visible_comic, :author_id => @author.id
      c = Comic.list
      c.should eq [v, @comic]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @comic2 = Factory :visible_comic, :author_id => @author.id
        @comic3 = Factory :editable_comic, :author_id => @author.id
        @comic4 = Factory :normal_comic, :author_id => @author.id
        @comic5 = Factory :editable_comic, :author_id => @author.id
        Comic.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        c = Comic.list
        c.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        c = Comic.list({}, 1)
        c.should eq [@comic5, @comic4]
      end
      it 'page=2なら中間2件を返す' do
        c = Comic.list({}, 2)
        c.should eq [@comic3, @comic2]
      end
      it 'page=3なら先頭1件を返す' do
        c = Comic.list({}, 3)
        c.should eq [@comic]
      end
    end
  end
end
